--- apps/app_chanspy.c.orig	2009-04-15 18:48:25.000000000 -0400
+++ apps/app_chanspy.c	2009-04-15 19:43:19.000000000 -0400
@@ -73,6 +73,7 @@
 "    b             - Only spy on channels involved in a bridged call.\n"
 "    g(grp)        - Match only channels where their ${SPYGROUP} variable is set to\n"
 "                    contain 'grp' in an optional : delimited list.\n"
+"    h             - Hang up when the spied-on channel hangs up.\n"
 "    q             - Don't play a beep when beginning to spy on a channel, or speak the\n"
 "                    selected channel name.\n"
 "    r[(basename)] - Record the session to the monitor spool directory. An\n"
@@ -123,6 +124,7 @@
 	OPTION_RECORD    = (1 << 4),
 	OPTION_WHISPER	 = (1 << 5),
 	OPTION_PRIVATE   = (1 << 6),	/* Private Whisper mode */
+	OPTION_HANGUP	 = (1 << 7),	/* Hang up when the spied-on channel hangs up. */
 } chanspy_opt_flags;
 
 enum {
@@ -135,6 +137,7 @@
 AST_APP_OPTIONS(spy_opts, {
 	AST_APP_OPTION('q', OPTION_QUIET),
 	AST_APP_OPTION('b', OPTION_BRIDGED),
+	AST_APP_OPTION('h', OPTION_HANGUP),
 	AST_APP_OPTION('w', OPTION_WHISPER),
 	AST_APP_OPTION('W', OPTION_PRIVATE),
 	AST_APP_OPTION_ARG('v', OPTION_VOLUME, OPT_ARG_VOLUME),
@@ -495,6 +498,7 @@
 	char *ptr;
 	int num;
 	int num_spyed_upon = 1;
+	int hungup = 0;
 	struct chanspy_ds chanspy_ds;
 
 	ast_mutex_init(&chanspy_ds.lock);
@@ -648,8 +652,16 @@
 				}
 
 				peer = NULL;
+			} else if ( res == 0 && ast_test_flag(flags, OPTION_HANGUP)) {
+				hungup=1; //to break the outter loop
+				break;
 			}
 		}
+
+		if (hungup) {
+			break;
+		}
+
 		if (res == -1 || ast_check_hangup(chan))
 			break;
 	}
@@ -660,6 +672,12 @@
 
 	ast_mutex_destroy(&chanspy_ds.lock);
 
+	if (ast_test_flag(flags, OPTION_HANGUP)) {
+		if (option_verbose >= 2) {
+			ast_verbose(VERBOSE_PREFIX_3 "Hang up due to the spied-on channel hanging up.\n");
+		}
+	} 
+
 	return res;
 }
 
